@@ -26,6 +26,7 @@
 #include "ephy-embed-shell.h"
 #include "ephy-history-types.h"
 #include "ephy-security-levels.h"
+#include "ephy-web-extension-proxy.h"
 
 G_BEGIN_DECLS
 
@@ -40,8 +41,7 @@ G_DECLARE_FINAL_TYPE (EphyWebView, ephy_web_view, EPHY, WEB_VIEW, WebKitWebView)
                                         "^https?://[^/\\.[:space:]]+.*$|" \
                                         "^about:.*$|" \
                                         "^data:.*$|" \
-                                        "^file:.*$|" \
-                                        "^inspector://.*$" \
+                                        "^file:.*$" \
                                         ")"
 
 #define EPHY_WEB_VIEW_DOMAIN_REGEX "^localhost(\\.[^[:space:]]+)?(:\\d+)?(:[0-9]+)?(/.*)?$|" \
@@ -70,6 +70,9 @@ typedef enum {
   EPHY_WEB_VIEW_ERROR_UNSAFE_BROWSING
 } EphyWebViewErrorPage;
 
+typedef void (* EphyPasswordSaveRequestCallback) (int      response_id,
+                                                  gpointer data);
+
 GType                      ephy_web_view_chrome_get_type          (void);
 GType                      ephy_web_view_security_level_get_type  (void);
 GtkWidget *                ephy_web_view_new                      (void);
@@ -158,4 +161,13 @@ gboolean                   ephy_web_view_is_reader_mode_available (EphyWebView
 
 gboolean                   ephy_web_view_get_reader_mode_state    (EphyWebView               *view);
 
+EphyWebExtensionProxy     *ephy_web_view_get_web_extension_proxy  (EphyWebView               *view);
+
+void                       ephy_web_view_show_auth_form_save_request (EphyWebView                    *web_view,
+                                                                      const char                     *origin,
+                                                                      const char                     *username,
+                                                                      EphyPasswordSaveRequestCallback response_callback,
+                                                                      gpointer                        response_data,
+                                                                      GDestroyNotify                  response_destroy);
+
 G_END_DECLS
