@@ -56,7 +56,6 @@ struct _EphyShell {
   GList *windows;
   GObject *lockdown;
   EphyBookmarksManager *bookmarks_manager;
-  EphyPasswordManager *password_manager;
   EphyHistoryManager *history_manager;
   EphyOpenTabsManager *open_tabs_manager;
   GNetworkMonitor *network_monitor;
@@ -270,16 +269,17 @@ static GActionEntry app_entries[] = {
   { "quit", quit_application, NULL, NULL, NULL },
 };
 
+static GActionEntry non_incognito_extra_app_entries[] = {
+  { "reopen-closed-tab", reopen_closed_tab, NULL, NULL, NULL },
+};
+
 static GActionEntry app_mode_app_entries[] = {
+  { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
 };
 
-static GActionEntry app_normal_mode_entries[] = {
-  { "reopen-closed-tab", reopen_closed_tab, NULL, NULL, NULL },
-};
-
 static void
 download_started_cb (WebKitWebContext *web_context,
                      WebKitDownload   *download,
@@ -330,7 +330,7 @@ register_synchronizable_managers (EphyShell       *shell,
   }
 
   if (ephy_sync_utils_passwords_sync_is_enabled ()) {
-    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_password_manager (shell));
+    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL (shell)));
     ephy_sync_service_register_manager (service, manager);
   }
 
@@ -376,13 +376,22 @@ sync_secrets_load_finished_cb (EphySyncService *service,
   ephy_sync_service_start_sync (service);
 }
 
+static void
+set_accel_for_action (EphyShell   *shell,
+                      const gchar *detailed_action_name,
+                      const gchar *accel)
+{
+  const char *accels[] = { accel, NULL };
+
+  gtk_application_set_accels_for_action (GTK_APPLICATION (shell), detailed_action_name, accels);
+}
+
 static void
 ephy_shell_startup (GApplication *application)
 {
   EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (application);
   EphyShell *shell = EPHY_SHELL (application);
   EphyEmbedShellMode mode;
-  GtkBuilder *builder;
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->startup (application);
 
@@ -392,11 +401,6 @@ ephy_shell_startup (GApplication *application)
                     G_CALLBACK (download_started_cb),
                     application);
 
-  builder = gtk_builder_new ();
-  gtk_builder_add_from_resource (builder,
-                                 "/org/gnome/epiphany/gtk/application-menu.ui",
-                                 NULL);
-
   mode = ephy_embed_shell_get_mode (embed_shell);
   if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION) {
     g_action_map_add_action_entries (G_ACTION_MAP (application),
@@ -405,7 +409,7 @@ ephy_shell_startup (GApplication *application)
 
     if (mode != EPHY_EMBED_SHELL_MODE_INCOGNITO) {
       g_action_map_add_action_entries (G_ACTION_MAP (application),
-                                       app_normal_mode_entries, G_N_ELEMENTS (app_normal_mode_entries),
+                                       non_incognito_extra_app_entries, G_N_ELEMENTS (non_incognito_extra_app_entries),
                                        application);
       g_object_bind_property (G_OBJECT (ephy_shell_get_session (shell)),
                               "can-undo-tab-closed",
@@ -418,19 +422,26 @@ ephy_shell_startup (GApplication *application)
         /* Create the sync service. */
         ephy_shell_get_sync_service (shell);
       }
-    }
 
-    gtk_application_set_app_menu (GTK_APPLICATION (application),
-                                  G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
+      /* Actions that are disabled in app mode */
+      set_accel_for_action (shell, "app.new-window", "<Primary>n");
+      set_accel_for_action (shell, "app.new-incognito", "<Primary><Shift>n");
+      set_accel_for_action (shell, "app.reopen-closed-tab", "<Primary><Shift>t");
+      set_accel_for_action (shell, "app.import-bookmarks", "<Primary><Shift>m");
+      set_accel_for_action (shell, "app.export-bookmarks", "<Primary><Shift>x");
+      set_accel_for_action (shell, "app.shortcuts", "<Primary>F1");
+      set_accel_for_action (shell, "app.help", "F1");
+    }
   } else {
     g_action_map_add_action_entries (G_ACTION_MAP (application),
                                      app_mode_app_entries, G_N_ELEMENTS (app_mode_app_entries),
                                      application);
-    gtk_application_set_app_menu (GTK_APPLICATION (application),
-                                  G_MENU_MODEL (gtk_builder_get_object (builder, "app-mode-app-menu")));
   }
 
-  g_object_unref (builder);
+  /* Actions that are available in both app mode and browser mode */
+  set_accel_for_action (shell, "app.history", "<Primary>h");
+  set_accel_for_action (shell, "app.preferences", "<Primary>e");
+  set_accel_for_action (shell, "app.quit", "<Primary>q");
 }
 
 static void
@@ -693,7 +704,6 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->network_monitor);
   g_clear_object (&shell->sync_service);
   g_clear_object (&shell->bookmarks_manager);
-  g_clear_object (&shell->password_manager);
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
 
@@ -896,25 +906,6 @@ ephy_shell_get_bookmarks_manager (EphyShell *shell)
   return shell->bookmarks_manager;
 }
 
-/**
- * ephy_shell_get_password_manager:
- * @shell: the #EphyShell
- *
- * Returns the passwords manager.
- *
- * Return value: (transfer none): An #EphyPasswordManager.
- */
-EphyPasswordManager *
-ephy_shell_get_password_manager (EphyShell *shell)
-{
-  g_assert (EPHY_IS_SHELL (shell));
-
-  if (shell->password_manager == NULL)
-    shell->password_manager = ephy_password_manager_new ();
-
-  return shell->password_manager;
-}
-
 /**
  * ephy_shell_get_history_manager:
  * @shell: the #EphyShell
