@@ -33,7 +33,8 @@ struct _EphySuggestionModel {
   EphyHistoryService   *history_service;
   EphyBookmarksManager *bookmarks_manager;
   GSequence            *items;
-  GSList               *search_terms;
+  gchar               **search_terms;
+  GCancellable         *icon_cancellable;
 };
 
 enum {
@@ -48,7 +49,7 @@ static void list_model_iface_init (GListModelInterface *iface);
 G_DEFINE_TYPE_WITH_CODE (EphySuggestionModel, ephy_suggestion_model, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, list_model_iface_init))
 
-static GParamSpec *properties [N_PROPS];
+static GParamSpec *properties[N_PROPS];
 
 static void
 ephy_suggestion_model_finalize (GObject *object)
@@ -59,7 +60,10 @@ ephy_suggestion_model_finalize (GObject *object)
   g_clear_object (&self->history_service);
   g_clear_pointer (&self->items, g_sequence_free);
 
-  g_slist_free_full (self->search_terms, (GDestroyNotify)g_regex_unref);
+  g_cancellable_cancel (self->icon_cancellable);
+  g_clear_object (&self->icon_cancellable);
+
+  g_strfreev (self->search_terms);
 
   G_OBJECT_CLASS (ephy_suggestion_model_parent_class)->finalize (object);
 }
@@ -189,82 +193,11 @@ static void
 update_search_terms (EphySuggestionModel *self,
                      const char          *text)
 {
-  const char *current;
-  const char *ptr;
-  char *tmp;
-  char *term;
-  GRegex *term_regex;
-  GRegex *quote_regex;
-  gint count;
-  gboolean inside_quotes = FALSE;
-
   g_assert (EPHY_IS_SUGGESTION_MODEL (self));
 
-  if (self->search_terms) {
-    g_slist_free_full (self->search_terms, (GDestroyNotify)g_regex_unref);
-    self->search_terms = NULL;
-  }
+  g_strfreev (self->search_terms);
 
-  quote_regex = g_regex_new ("\"", G_REGEX_OPTIMIZE,
-                             G_REGEX_MATCH_NOTEMPTY, NULL);
-
-  /*
-   * This code loops through the string using pointer arythmetics.
-   * Although the string we are handling may contain UTF-8 chars
-   * this works because only ASCII chars affect what is actually
-   * copied from the string as a search term.
-   */
-  for (count = 0, current = ptr = text; ptr[0] != '\0'; ptr++, count++) {
-    /*
-     * If we found a double quote character; we will
-     * consume bytes up until the next quote, or
-     * end of line;
-     */
-    if (ptr[0] == '"')
-      inside_quotes = !inside_quotes;
-
-    /*
-     * If we found a space, and we are not looking for a
-     * closing double quote, or if the next char is the
-     * end of the string, append what we have already as
-     * a search term.
-     */
-    if (((ptr[0] == ' ') && (!inside_quotes)) || ptr[1] == '\0') {
-      /*
-       * We special-case the end of the line because
-       * we would otherwise not copy the last character
-       * of the search string, since the for loop will
-       * stop before that.
-       */
-      if (ptr[1] == '\0')
-        count++;
-
-      /*
-       * remove quotes, and quote any regex-sensitive
-       * characters
-       */
-      tmp = g_regex_escape_string (current, count);
-      term = g_regex_replace (quote_regex, tmp, -1, 0,
-                              "", G_REGEX_MATCH_NOTEMPTY, NULL);
-      g_strstrip (term);
-      g_free (tmp);
-
-      /* we don't want empty search terms */
-      if (term[0] != '\0') {
-        term_regex = g_regex_new (term,
-                                  G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
-                                  G_REGEX_MATCH_NOTEMPTY, NULL);
-        self->search_terms = g_slist_append (self->search_terms, term_regex);
-      }
-      g_free (term);
-
-      /* count will be incremented by the for loop */
-      count = -1;
-      current = ptr + 1;
-    }
-  }
-
-  g_regex_unref (quote_regex);
+  self->search_terms = g_strsplit (text, " ", -1);
 }
 
 static gboolean
@@ -276,13 +209,13 @@ should_add_bookmark_to_model (EphySuggestionModel *self,
   gboolean ret = TRUE;
 
   if (self->search_terms) {
-    GSList *iter;
-    GRegex *current = NULL;
+    guint len = g_strv_length (self->search_terms);
+    guint i;
 
-    for (iter = self->search_terms; iter != NULL; iter = iter->next) {
-      current = (GRegex *)iter->data;
-      if ((!g_regex_match (current, title ? title : "", G_REGEX_MATCH_NOTEMPTY, NULL)) &&
-          (!g_regex_match (current, location ? location : "", G_REGEX_MATCH_NOTEMPTY, NULL))) {
+    for (i = 0; i < len; i++) {
+      gchar *str = self->search_terms[i];
+
+      if (!strstr (str, title ? title : "") && !strstr (str, location ? location : "")) {
         ret = FALSE;
         break;
       }
@@ -292,6 +225,51 @@ should_add_bookmark_to_model (EphySuggestionModel *self,
   return ret;
 }
 
+static void
+icon_loaded_cb (GObject      *source,
+                GAsyncResult *result,
+                gpointer      user_data)
+{
+  WebKitFaviconDatabase *database = WEBKIT_FAVICON_DATABASE (source);
+  EphySuggestion *suggestion;
+  GError *error = NULL;
+  cairo_surface_t *favicon;
+  gdouble x_scale, y_scale;
+  int x, y;
+
+  favicon = webkit_favicon_database_get_favicon_finish (database, result, &error);
+
+  if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) || favicon == NULL)
+    return;
+
+  suggestion = EPHY_SUGGESTION (user_data);
+
+  x = cairo_image_surface_get_width (favicon);
+  y = cairo_image_surface_get_height (favicon);
+  x_scale = (gdouble)x / 16;
+  y_scale = (gdouble)y / 16;
+
+  cairo_surface_set_device_scale (favicon, x_scale, y_scale);
+
+  ephy_suggestion_set_favicon (suggestion, favicon);
+}
+
+static void
+load_favicon (EphySuggestionModel *model,
+              EphySuggestion      *suggestion,
+              const char          *url)
+{
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+  WebKitWebContext *context = ephy_embed_shell_get_web_context (shell);
+  WebKitFaviconDatabase *database = webkit_web_context_get_favicon_database (context);
+
+  webkit_favicon_database_get_favicon (database,
+                                       url,
+                                       model->icon_cancellable,
+                                       icon_loaded_cb,
+                                       suggestion);
+}
+
 static guint
 add_bookmarks (EphySuggestionModel *self,
                const char          *query)
@@ -316,6 +294,8 @@ add_bookmarks (EphySuggestionModel *self,
       EphySuggestion *suggestion;
 
       suggestion = ephy_suggestion_new (title, url);
+      load_favicon (self, suggestion, url);
+
       g_sequence_append (self->items, suggestion);
       added++;
     }
@@ -326,7 +306,8 @@ add_bookmarks (EphySuggestionModel *self,
 
 static guint
 add_history (EphySuggestionModel *self,
-             GList               *urls)
+             GList               *urls,
+             const char          *query)
 {
   guint added = 0;
 
@@ -335,6 +316,8 @@ add_history (EphySuggestionModel *self,
     EphySuggestion *suggestion;
 
     suggestion = ephy_suggestion_new (url->title, url->url);
+    load_favicon (self, suggestion, url->url);
+
     g_sequence_append (self->items, suggestion);
     added++;
   }
@@ -361,6 +344,7 @@ add_search_engines (EphySuggestionModel *self,
 
     address = ephy_search_engine_manager_build_search_address (manager, engines[i], query);
     suggestion = ephy_suggestion_new_without_subtitle (engines[i], address);
+    load_favicon (self, suggestion, address);
 
     g_sequence_append (self->items, suggestion);
     added++;
@@ -384,20 +368,27 @@ query_completed_cb (EphyHistoryService *service,
   const gchar *query;
   GList *urls;
   guint removed;
-  guint added;
+  guint added = 0;
 
   self = g_task_get_source_object (task);
   query = g_task_get_task_data (task);
   urls = (GList *)result_data;
 
+  g_cancellable_cancel (self->icon_cancellable);
+  g_clear_object (&self->icon_cancellable);
+
+  self->icon_cancellable = g_cancellable_new ();
+
   removed = g_sequence_get_length (self->items);
 
   g_clear_pointer (&self->items, g_sequence_free);
   self->items = g_sequence_new (g_object_unref);
 
-  added = add_bookmarks (self, query);
-  added += add_history (self, urls);
-  added += add_search_engines (self, query);
+  if (strlen (query) > 0) {
+    added = add_bookmarks (self, query);
+    added += add_history (self, urls, query);
+    added += add_search_engines (self, query);
+  }
 
   g_list_model_items_changed (G_LIST_MODEL (self), 0, removed, added);
 
