@@ -33,6 +33,7 @@
 #include "ephy-flatpak-utils.h"
 #include "ephy-gui.h"
 #include "ephy-langs.h"
+#include "ephy-lib-type-builtins.h"
 #include "ephy-prefs.h"
 #include "ephy-search-engine-dialog.h"
 #include "ephy-session.h"
@@ -93,7 +94,7 @@ struct _PrefsDialog {
   GtkWidget *enable_plugins_checkbutton;
   GtkWidget *enable_safe_browsing_checkbutton;
 
-  /* fonts */
+  /* fonts & style */
   GtkWidget *use_gnome_fonts_checkbutton;
   GtkWidget *custom_fonts_table;
   GtkWidget *sans_fontbutton;
@@ -101,6 +102,9 @@ struct _PrefsDialog {
   GtkWidget *mono_fontbutton;
   GtkWidget *css_checkbox;
   GtkWidget *css_edit_button;
+  GtkWidget *reader_mode_box;
+  GtkWidget *reader_mode_font_style;
+  GtkWidget *reader_mode_color_scheme;
 
   /* stored data */
   GtkWidget *always;
@@ -204,7 +208,7 @@ sync_collection_toggled_cb (GtkToggleButton *button,
   if (GTK_WIDGET (button) == dialog->sync_bookmarks_checkbutton) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_bookmarks_manager (shell));
   } else if (GTK_WIDGET (button) == dialog->sync_passwords_checkbutton) {
-    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_password_manager (shell));
+    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL (shell)));
   } else if (GTK_WIDGET (button) == dialog->sync_history_checkbutton) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_history_manager (shell));
   } else if (GTK_WIDGET (button) == dialog->sync_open_tabs_checkbutton) {
@@ -906,7 +910,7 @@ on_manage_passwords_button_clicked (GtkWidget   *button,
   EphyPasswordsDialog *passwords_dialog;
   EphyPasswordManager *password_manager;
 
-  password_manager = ephy_shell_get_password_manager (ephy_shell_get_default ());
+  password_manager = ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL(ephy_shell_get_default ()));
   passwords_dialog = ephy_passwords_dialog_new (password_manager);
 
   gtk_window_set_transient_for (GTK_WINDOW (passwords_dialog), GTK_WINDOW (dialog));
@@ -960,7 +964,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_button_label);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_box);
 
-  /* fonts */
+  /* fonts & style */
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, use_gnome_fonts_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, custom_fonts_table);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sans_fontbutton);
@@ -968,6 +972,9 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, mono_fontbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_checkbox);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_edit_button);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_box);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_font_style);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_color_scheme);
 
   /* stored data */
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, always);
@@ -1986,12 +1993,124 @@ setup_general_page (PrefsDialog *dialog)
     create_download_path_button (dialog);
 }
 
+enum {
+  ENUM_MODEL_COLUMN_LABEL,
+  ENUM_MODEL_COLUMN_NICK,
+  ENUM_MODEL_N_COLUMNS,
+};
+
+typedef struct {
+  gint        value;
+  const char *label;
+} EnumModelItem;
+
+static GtkTreeModel *
+enum_model_new (GType                enum_type,
+                gsize                n_items,
+                const EnumModelItem *items)
+{
+  GtkListStore *model;
+  GEnumClass *enum_class;
+  gsize i;
+
+  enum_class = g_type_class_ref (enum_type);
+  model = gtk_list_store_new (ENUM_MODEL_N_COLUMNS,
+                              G_TYPE_STRING,
+                              G_TYPE_STRING);
+
+  for (i = 0; i < n_items; i++) {
+    GtkTreeIter iter;
+    const char *nick;
+    guint  j;
+
+    nick = NULL;
+    for (j = 0; j < enum_class->n_values; j++) {
+      if (enum_class->values[j].value == items[i].value) {
+        nick = enum_class->values[j].value_nick;
+        break;
+      }
+    }
+    g_assert (nick != NULL);
+
+    gtk_list_store_append (model, &iter);
+    gtk_list_store_set (model,
+                        &iter,
+                        ENUM_MODEL_COLUMN_LABEL, _(items[i].label),
+                        ENUM_MODEL_COLUMN_NICK, nick,
+                        -1);
+  }
+
+  g_type_class_unref (enum_class);
+  return GTK_TREE_MODEL (model);
+}
+
+static void
+enum_model_attach (GtkTreeModel *model,
+                   GtkComboBox  *combobox,
+                   GSettings    *settings,
+                   const char   *key)
+{
+  GtkCellRenderer *renderer;
+
+  gtk_cell_layout_clear (GTK_CELL_LAYOUT (combobox));
+
+  gtk_combo_box_set_model (combobox, model);
+  gtk_combo_box_set_id_column (combobox,
+                               ENUM_MODEL_COLUMN_NICK);
+
+  g_settings_bind (settings,
+                   key,
+                   combobox,
+                   "active-id",
+                   G_SETTINGS_BIND_DEFAULT);
+
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox),
+                              renderer,
+                              FALSE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combobox),
+                                 renderer,
+                                 "text",
+                                 ENUM_MODEL_COLUMN_LABEL);
+}
+
+static const EnumModelItem reader_font_style_items[] = {
+  { EPHY_PREFS_READER_FONT_STYLE_SANS,  N_("Sans")  },
+  { EPHY_PREFS_READER_FONT_STYLE_SERIF, N_("Serif") },
+};
+
+static const EnumModelItem reader_color_scheme_items[] = {
+  { EPHY_PREFS_READER_COLORS_LIGHT, N_("Light") },
+  { EPHY_PREFS_READER_COLORS_DARK,  N_("Dark")  },
+};
+
 static void
 setup_fonts_page (PrefsDialog *dialog)
 {
   GSettings *web_settings;
+  GSettings *reader_settings;
+  GtkTreeModel *model;
 
   web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
+  reader_settings = ephy_settings_get (EPHY_PREFS_READER_SCHEMA);
+
+  model = enum_model_new (EPHY_TYPE_PREFS_READER_FONT_STYLE,
+                          G_N_ELEMENTS (reader_font_style_items),
+                          reader_font_style_items);
+  enum_model_attach (model,
+                     GTK_COMBO_BOX (dialog->reader_mode_font_style),
+                     reader_settings,
+                     EPHY_PREFS_READER_FONT_STYLE);
+  g_object_unref (model);
+
+  model = enum_model_new (EPHY_TYPE_PREFS_READER_COLOR_SCHEME,
+                          G_N_ELEMENTS (reader_color_scheme_items),
+                          reader_color_scheme_items);
+  enum_model_attach (model,
+                     GTK_COMBO_BOX (dialog->reader_mode_color_scheme),
+                     reader_settings,
+                     EPHY_PREFS_READER_COLOR_SCHEME);
+  g_object_unref (model);
 
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_USE_GNOME_FONTS,
@@ -2029,6 +2148,7 @@ setup_fonts_page (PrefsDialog *dialog)
                    dialog->css_edit_button,
                    "sensitive",
                    G_SETTINGS_BIND_GET);
+
   g_signal_connect (dialog->css_edit_button,
                     "clicked",
                     G_CALLBACK (css_edit_button_clicked_cb),
@@ -2232,6 +2352,8 @@ prefs_dialog_init (PrefsDialog *dialog)
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->do_not_track_checkbutton,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
+  gtk_widget_set_visible (dialog->reader_mode_box,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
 
   setup_general_page (dialog);
   setup_fonts_page (dialog);
