@@ -30,6 +30,7 @@
 #include "ephy-search-engine-manager.h"
 #include "ephy-settings.h"
 #include "ephy-sqlite-connection.h"
+#include "ephy-string.h"
 #include "ephy-sync-debug.h"
 #include "ephy-sync-utils.h"
 #include "ephy-uri-tester-shared.h"
@@ -86,7 +87,7 @@ migrate_app_desktop_file_categories (void)
 
     file = g_key_file_new ();
 
-    app_path = ephy_web_application_get_profile_directory (app->name);
+    app_path = ephy_web_application_get_profile_directory (app->id);
     desktop_file_path = g_build_filename (app_path, app->desktop_file, NULL);
     g_key_file_load_from_file (file, desktop_file_path, G_KEY_FILE_NONE, NULL);
 
@@ -610,6 +611,11 @@ migrate_search_engines (void)
   char *default_search_engine_address;
   const char *default_search_engine_name = _("Search the Web");
 
+  /* Search engine settings are only used in browser mode, so no need to migrate
+   * if we are not in browser mode. */
+  if (!ephy_dot_dir_is_default ())
+    return;
+
   bookmarks_manager = ephy_bookmarks_manager_new ();
   search_engine_manager = ephy_search_engine_manager_new ();
 
@@ -1187,6 +1193,28 @@ out:
   g_list_free_full (passwords, g_object_unref);
 }
 
+static void
+migrate_annoyance_list (void)
+{
+  GVariant *user_value;
+  const char **filters;
+  char **modified_filters;
+
+  /* Has the filters setting been modified? If not, we're done. */
+  user_value = g_settings_get_user_value (EPHY_SETTINGS_MAIN, EPHY_PREFS_ADBLOCK_FILTERS);
+  if (!user_value)
+    return;
+
+  /* The annoyance list was causing a bunch of problems. Forcibly remove it. */
+  filters = g_variant_get_strv (user_value, NULL);
+  modified_filters = ephy_strv_remove (filters, "https://easylist.to/easylist/fanboy-annoyance.txt");
+  g_settings_set_strv (EPHY_SETTINGS_MAIN, EPHY_PREFS_ADBLOCK_FILTERS, (const char * const *)modified_filters);
+
+  g_variant_unref (user_value);
+  g_free (filters);
+  g_strfreev (modified_filters);
+}
+
 static void
 migrate_nothing (void)
 {
@@ -1227,7 +1255,8 @@ const EphyProfileMigrator migrators[] = {
   /* 24 */ migrate_bookmarks_timestamp,
   /* 25 */ migrate_passwords_timestamp,
   /* 26 */ migrate_nothing,
-  /* 27 */ migrate_search_engines
+  /* 27 */ migrate_search_engines,
+  /* 28 */ migrate_annoyance_list
 };
 
 static gboolean
