@@ -2430,3 +2430,26 @@ window_cmd_change_show_tab_state (GSimpleAction *action,
 {
   /* This page intentionally left blank. */
 }
+
+void
+window_cmd_toggle_reader_mode (GSimpleAction *action,
+                               GVariant      *parameter,
+                               gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+  gboolean active;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  web_view = ephy_embed_get_web_view (embed);
+
+  if (!ephy_web_view_is_reader_mode_available (web_view))
+    return;
+
+  active = ephy_web_view_get_reader_mode_state (web_view);
+
+  ephy_web_view_toggle_reader_mode (web_view, !active);
+}
