@@ -46,9 +46,9 @@ server_callback (SoupServer        *server,
                  SoupClientContext *context,
                  gpointer           data)
 {
-  if (g_str_equal (path, "/cancelled"))
+  if (!strcmp (path, "/cancelled"))
     soup_message_set_status (msg, SOUP_STATUS_CANT_CONNECT);
-  else if (g_str_equal (path, "/redirect")) {
+  else if (!strcmp (path, "/redirect")) {
     soup_message_set_status (msg, SOUP_STATUS_MOVED_PERMANENTLY);
     soup_message_headers_append (msg->response_headers, "Location", "/redirect-result");
   } else
@@ -456,7 +456,7 @@ main (int argc, char *argv[])
   ephy_debug_init ();
 
   if (!ephy_file_helpers_init (NULL,
-                               EPHY_FILE_HELPERS_PRIVATE_PROFILE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
+                               EPHY_FILE_HELPERS_TESTING_MODE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
                                NULL)) {
     g_debug ("Something wrong happened with ephy_file_helpers_init()");
     return -1;
